@@ -220,34 +220,6 @@ class ApiParameters(Parameters):
 
 
 class ModuleParameters(Parameters):
-    def _policy_exists(self):
-        uri = 'https://{0}:{1}/mgmt/tm/asm/policies/'.format(
-            self.client.provider['server'],
-            self.client.provider['server_port'],
-        )
-        query = "?$filter=contains(name,'{0}')+and+contains(partition,'{1}')&$select=name,partition".format(
-            self.want.name, self.want.partition
-        )
-        resp = self.client.api.get(uri + query)
-        try:
-            response = resp.json()
-        except ValueError as ex:
-            raise F5ModuleError(str(ex))
-        if 'items' in response and response['items'] != []:
-            return True
-        return False
-
-    @property
-    def name(self):
-        if self._policy_exists():
-            return self._values['name']
-        else:
-            raise F5ModuleError(
-                "The specified ASM policy {0} on partition {1} does not exist on device.".format(
-                    self._values['name'], self._values['partition']
-                )
-            )
-
     @property
     def file(self):
         if self._values['file'] is not None:
@@ -372,7 +344,7 @@ class ModuleManager(object):
         changes = reportable.to_return()
         result.update(**changes)
         result.update(dict(changed=True))
-        send_teem(start, self.module, version)
+        send_teem(start, self.client, self.module, version)
         return result
 
     def export(self):
@@ -386,7 +358,7 @@ class ModuleManager(object):
             raise F5ModuleError(
                 "File '{0}' already exists.".format(self.want.fulldest)
             )
-        self.execute()
+        self.create()
 
     def create(self):
         self._set_changed_options()
@@ -419,11 +391,42 @@ class ModuleManager(object):
         return True
 
     def exists(self):
+        self.policy_exists()
         if not self.want.inline:
             if os.path.exists(self.want.fulldest):
                 return True
         return False
 
+    def policy_exists(self):
+        uri = 'https://{0}:{1}/mgmt/tm/asm/policies/'.format(
+            self.client.provider['server'],
+            self.client.provider['server_port'],
+        )
+        query = "?$filter=contains(name,'{0}')+and+contains(partition,'{1}')&$select=name,partition".format(
+            self.want.name, self.want.partition
+        )
+        resp = self.client.api.get(uri + query)
+        try:
+            response = resp.json()
+        except ValueError as ex:
+            raise F5ModuleError(str(ex))
+
+        if resp.status not in [200, 201] or 'code' in response and response['code'] not in [200, 201]:
+            raise F5ModuleError(resp.content)
+
+        if 'items' in response and response['items'] != []:
+            if len(response['items']) == 1:
+                return True
+            else:
+                for item in response['items']:
+                    if item['name'] == self.want.name:
+                        return True
+        raise F5ModuleError(
+            "The specified ASM policy {0} on partition {1} does not exist on device.".format(
+                self.want.name, self.want.partition
+            )
+        )
+
     def create_on_device(self):
         self._set_policy_link()
         params = self.changes.api_params()
@@ -494,9 +497,13 @@ class ModuleManager(object):
             response = resp.json()
         except ValueError as ex:
             raise F5ModuleError(str(ex))
-
         if 'items' in response and response['items'] != []:
-            policy_link = response['items'][0]['selfLink']
+            if len(response['items']) == 1:
+                policy_link = response['items'][0]['selfLink']
+            else:
+                for item in response['items']:
+                    if item['name'] == self.want.name:
+                        policy_link = item['selfLink']
 
         if not policy_link:
             raise F5ModuleError("The policy was not found")
@@ -528,10 +535,45 @@ class ModuleManager(object):
         if resp.status not in [200, 201] or 'code' in response and response['code'] not in [200, 201]:
             raise F5ModuleError(resp.content)
 
+        self._stat_binary_on_device()
         self._move_binary_to_download()
 
         return True
 
+    def _stat_binary_on_device(self):
+        params = dict(
+            command='run',
+            utilCmdArgs='/var/tmp/{0} -l'.format(self.want.file)
+        )
+
+        uri = "https://{0}:{1}/mgmt/tm/util/unix-ls/".format(
+            self.client.provider['server'],
+            self.client.provider['server_port'],
+        )
+        resp = self.client.api.post(uri, json=params)
+
+        try:
+            response = resp.json()
+        except ValueError as ex:
+            raise F5ModuleError(str(ex))
+
+        if resp.status not in [200, 201] or 'code' in response and response['code'] not in [200, 201]:
+            raise F5ModuleError(resp.content)
+
+        if 'commandResult' not in response:
+            raise F5ModuleError("Failed to obtain file information, aborting.")
+
+        if 'Error' in response['commandResult'] or 'error' in response['commandResult']:
+            raise F5ModuleError(response['commandResult'])
+
+        if '/var/tmp/{0}'.format(self.want.file) not in response['commandResult']:
+            raise F5ModuleError("Cannot get size of exported binary file, aborting")
+
+        size = response['commandResult']
+
+        self.want.file_size = int(size.split()[4])
+        return True
+
     def _move_binary_to_download(self):
         name = '{0}~{1}'.format(self.client.provider['user'], self.want.file)
         move_path = '/var/tmp/{0} {1}/{2}'.format(
