@@ -47,13 +47,16 @@ try:
 except ImportError:
     ansible_version = 'unknown'
 
-COLLECTION_VERSION = "21.1.1"
+COLLECTION_VERSION = "21.6.0"
+CLIENT_APP_VERSION = "%s/" + COLLECTION_VERSION
+IMPORT_EXCEPTION = None
 
 try:
     from netapp_lib.api.zapi import zapi
     HAS_NETAPP_LIB = True
-except ImportError:
+except ImportError as exc:
     HAS_NETAPP_LIB = False
+    IMPORT_EXCEPTION = exc
 
 try:
     import requests
@@ -107,6 +110,10 @@ def has_netapp_lib():
     return HAS_NETAPP_LIB
 
 
+def netapp_lib_is_required():
+    return "Error: the python NetApp-Lib module is required.  Import error: %s" % str(IMPORT_EXCEPTION)
+
+
 def has_sf_sdk():
     return HAS_SF_SDK
 
@@ -124,7 +131,7 @@ def na_ontap_host_argument_spec():
         use_rest=dict(required=False, type='str', default='auto'),
         feature_flags=dict(required=False, type='dict', default=dict()),
         cert_filepath=dict(required=False, type='str'),
-        key_filepath=dict(required=False, type='str'),
+        key_filepath=dict(required=False, type='str', no_log=False),
     )
 
 
@@ -147,7 +154,8 @@ def get_feature(module, feature_name):
         sanitize_code_points=[8],               # unicode values, 8 is backspace
         show_modified=True,
         always_wrap_zapi=True,                  # for better error reporting
-        trace_apis=False                        # if true, append ZAPI and REST requests/responses to /tmp/ontap_zapi.txt
+        trace_apis=False,                       # if true, append ZAPI and REST requests/responses to /tmp/ontap_zapi.txt
+        flexcache_delete_return_timeout=5       # ONTAP bug if too big?
     )
 
     if module.params['feature_flags'] is not None and feature_name in module.params['feature_flags']:
@@ -261,7 +269,7 @@ def setup_na_ontap_zapi(module, vserver=None, wrap_zapi=False):
         server.set_server_type('FILER')
         return server
     else:
-        module.fail_json(msg="the python NetApp-Lib module is required")
+        module.fail_json(msg=netapp_lib_is_required())
 
 
 def is_zapi_connection_error(message):
@@ -455,8 +463,9 @@ if HAS_NETAPP_LIB:
         def _create_request(self, na_element, enable_tunneling=False):
             ''' intercept newly created request to add Authorization header '''
             request, netapp_element = super(OntapZAPICx, self)._create_request(na_element, enable_tunneling=enable_tunneling)
+            request.add_header('X-Dot-Client-App', CLIENT_APP_VERSION % self.module._name)
             if self.base64_creds is not None:
-                request.add_header("Authorization", "Basic %s" % self.base64_creds)
+                request.add_header('Authorization', 'Basic %s' % self.base64_creds)
             return request, netapp_element
 
         # as is from latest version of netapp-lib
@@ -534,7 +543,7 @@ class OntapRestAPI(object):
         self.auth_method = set_auth_method(self.module, self.username, self.password, self.cert_filepath, self.key_filepath)
         self.check_required_library()
         if has_feature(module, 'trace_apis'):
-            logging.basicConfig(filename='/tmp/ontap_apis.log', level=logging.DEBUG)
+            logging.basicConfig(filename='/tmp/ontap_apis.log', level=logging.DEBUG, format='%(asctime)s %(levelname)-8s %(message)s')
 
     def requires_ontap_9_6(self, module_name):
         self.requires_ontap_version(module_name)
@@ -543,18 +552,44 @@ class OntapRestAPI(object):
         suffix = " - %s" % self.is_rest_error if self.is_rest_error is not None else ""
         return "%s only supports REST, and requires ONTAP %s or later.%s" % (module_name, version, suffix)
 
-    def options_require_ontap_version(self, options, version='9.6'):
+    def options_require_ontap_version(self, options, version='9.6', use_rest=None):
+        current_version = self.get_ontap_version()
         suffix = " - %s" % self.is_rest_error if self.is_rest_error is not None else ""
-        if isinstance(options, list) and len(options) > 1:
-            tag = "any of %s" % options
+        if current_version != (-1, -1):
+            suffix += " - ONTAP version: %s.%s" % current_version
+        if use_rest is not None:
+            suffix += " - using %s" % ('REST' if use_rest else 'ZAPI')
+        if isinstance(options, list):
+            if len(options) > 1:
+                tag = "any of %s" % options
+            elif len(options) == 1:
+                tag = str(options[0])
+            else:
+                tag = str(options)
         else:
             tag = str(options)
-        return 'using %s requires ONTAP %s or later and REST must be enabled.%s' % (tag, version, suffix)
+        return 'using %s requires ONTAP %s or later and REST must be enabled%s.' % (tag, version, suffix)
+
+    def meets_rest_minimum_version(self, use_rest, minimum_generation, minimum_major):
+        return use_rest and self.get_ontap_version() >= (minimum_generation, minimum_major)
 
     def check_required_library(self):
         if not HAS_REQUESTS:
             self.module.fail_json(msg=missing_required_lib('requests'))
 
+    def build_headers(self, accept=None, vserver_name=None, vserver_uuid=None):
+        headers = dict()
+        headers['X-Dot-Client-App'] = CLIENT_APP_VERSION % self.module._name
+        # accept is used to turn on/off HAL linking
+        if accept is not None:
+            headers['accept'] = accept
+        # vserver tunneling using vserver name and/or UUID
+        if vserver_name is not None:
+            headers['X-Dot-SVM-Name'] = vserver_name
+        if vserver_uuid is not None:
+            headers['X-Dot-SVM-UUID'] = vserver_uuid
+        return headers
+
     def send_request(self, method, api, params, json=None, accept=None,
                      vserver_name=None, vserver_uuid=None):
         ''' send http request and process reponse, including error conditions '''
@@ -564,17 +599,7 @@ class OntapRestAPI(object):
         json_dict = None
         json_error = None
         error_details = None
-        headers = None
-        if accept is not None or vserver_name is not None or vserver_uuid is not None:
-            headers = dict()
-            # accept is used to turn on/off HAL linking
-            if accept is not None:
-                headers['accept'] = accept
-            # vserver tunneling using vserver name and/or UUID
-            if vserver_name is not None:
-                headers['X-Dot-SVM-Name'] = vserver_name
-            if vserver_uuid is not None:
-                headers['X-Dot-SVM-UUID'] = vserver_uuid
+        headers = self.build_headers(accept, vserver_name, vserver_uuid)
 
         def get_json(response):
             ''' extract json, and error message if present '''
@@ -675,9 +700,9 @@ class OntapRestAPI(object):
 
                 message = job_json.get('message', '')
                 if job_json['state'] == 'failure':
-                    # if the job as failed, return message as error
+                    # if the job has failed, return message as error
                     return None, message
-                if job_json['state'] != 'running':
+                if job_json['state'] not in ('queued', 'running'):
                     keep_running = False
                 else:
                     # Would like to post a message to user (not sure how)
@@ -726,45 +751,57 @@ class OntapRestAPI(object):
             try:
                 self.ontap_version[key] = version.get(key, -1)
             except AttributeError:
-                self.ontap_version[key] = 'unreadable version'
+                self.ontap_version[key] = -1
         self.ontap_version['valid'] = True
         for key in self.ontap_version:
-            if self.ontap_version == -1:
+            if self.ontap_version[key] == -1:
                 self.ontap_version['valid'] = False
                 break
 
     def get_ontap_version(self):
         if self.ontap_version['valid']:
             return self.ontap_version['generation'], self.ontap_version['major']
-        return None, None
+        return -1, -1
 
-    def _is_rest(self, used_unsupported_rest_properties=None):
-        if self.use_rest not in ['always', 'auto', 'never']:
-            error = "use_rest must be one of: never, always, auto. Got: '%s'" % self.use_rest
-            return False, error
-        if self.use_rest == "always":
-            if used_unsupported_rest_properties:
-                error = "REST API currently does not support '%s'" % \
-                        ', '.join(used_unsupported_rest_properties)
-                return True, error
-            else:
-                return True, None
-        if self.use_rest == 'never' or used_unsupported_rest_properties:
-            # force ZAPI if requested or if some parameter requires it
-            return False, None
-        # using GET rather than HEAD because the error messages are different
+    def get_ontap_version_using_rest(self):
+        # using GET rather than HEAD because the error messages are different,
+        # and we need the version as some REST options are not available in earlier versions
         method = 'GET'
         api = 'cluster'
         params = {'fields': ['version']}
         status_code, message, error = self.send_request(method, api, params=params)
         self.set_version(message)
         self.is_rest_error = str(error) if error else None
+        if error:
+            self.log_error(status_code, str(error))
+        return status_code
+
+    def _is_rest(self, used_unsupported_rest_properties=None):
+        if self.use_rest not in ['always', 'auto', 'never']:
+            error = "use_rest must be one of: never, always, auto. Got: '%s'" % self.use_rest
+            return False, error
+        if self.use_rest == "always" and used_unsupported_rest_properties:
+            error = "REST API currently does not support '%s'" % \
+                    ', '.join(used_unsupported_rest_properties)
+            return True, error
+        if self.use_rest == 'never':
+            # force ZAPI if requested
+            return False, None
+        status_code = self.get_ontap_version_using_rest()
+        if self.use_rest == 'always':
+            # ignore error, it will show up later when calling another REST API
+            return True, None
+        # we're now using 'auto'
+        if used_unsupported_rest_properties:
+            # force ZAPI if some parameter requires it
+            if self.get_ontap_version() > (9, 5):
+                self.module.warn('Falling back to ZAPI because of unsupported option(s) or option value(s) in REST: %s' % used_unsupported_rest_properties)
+            return False, None
         if self.get_ontap_version() in ((9, 4), (9, 5)):
             # we can't trust REST support on 9.5, and not at all on 9.4
             return False, None
         if status_code == 200:
             return True, None
-        self.log_error(status_code, str(error))
         return False, None
 
     def is_rest(self, used_unsupported_rest_properties=None):
@@ -775,6 +812,7 @@ class OntapRestAPI(object):
         return use_rest, error
 
     def log_error(self, status_code, message):
+        LOG.error("%s: %s", status_code, message)
         self.errors.append(message)
         self.debug_logs.append((status_code, message))
 
